#ifndef FULLSCREEN_SHELL_UNSTABLE_V1_CLIENT_PROTOCOL_H
#define FULLSCREEN_SHELL_UNSTABLE_V1_CLIENT_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-client.h"

struct wl_client;
struct wl_resource;

struct wl_output;
struct wl_surface;
struct zwp_fullscreen_shell_mode_feedback_v1;
struct zwp_fullscreen_shell_v1;

extern const struct wl_interface zwp_fullscreen_shell_v1_interface;
extern const struct wl_interface zwp_fullscreen_shell_mode_feedback_v1_interface;

#ifndef ZWP_FULLSCREEN_SHELL_V1_CAPABILITY_ENUM
#define ZWP_FULLSCREEN_SHELL_V1_CAPABILITY_ENUM
/**
 * zwp_fullscreen_shell_v1_capability - capabilities advertised by the
 *	compositor
 * @ZWP_FULLSCREEN_SHELL_V1_CAPABILITY_ARBITRARY_MODES: compositor is
 *	capable of almost any output mode
 * @ZWP_FULLSCREEN_SHELL_V1_CAPABILITY_CURSOR_PLANE: compositor has a
 *	separate cursor plane
 *
 * Various capabilities that can be advertised by the compositor. They
 * are advertised one-at-a-time when the wl_fullscreen_shell interface is
 * bound. See the wl_fullscreen_shell.capability event for more details.
 *
 * ARBITRARY_MODE: This is a hint to the client that indicates that the
 * compositor is capable of setting practically any mode on its outputs. If
 * this capability is provided,
 * wl_fullscreen_shell.present_surface_for_mode will almost never fail and
 * clients should feel free to set whatever mode they like. If the
 * compositor does not advertise this, it may still support some modes that
 * are not advertised through wl_global.mode but it is less likely.
 *
 * CURSOR_PLANE: This is a hint to the client that indicates that the
 * compositor can handle a cursor surface from the client without actually
 * compositing. This may be because of a hardware cursor plane or some
 * other mechanism. If the compositor does not advertise this capability
 * then setting wl_pointer.cursor may degrade performance or be ignored
 * entirely. If CURSOR_PLANE is not advertised, it is recommended that the
 * client draw its own cursor and set wl_pointer.cursor(NULL).
 */
enum zwp_fullscreen_shell_v1_capability {
	ZWP_FULLSCREEN_SHELL_V1_CAPABILITY_ARBITRARY_MODES = 1,
	ZWP_FULLSCREEN_SHELL_V1_CAPABILITY_CURSOR_PLANE = 2,
};
#endif /* ZWP_FULLSCREEN_SHELL_V1_CAPABILITY_ENUM */

#ifndef ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_ENUM
#define ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_ENUM
/**
 * zwp_fullscreen_shell_v1_present_method - different method to set the
 *	surface fullscreen
 * @ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_DEFAULT: no preference, apply
 *	default policy
 * @ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_CENTER: center the surface on
 *	the output
 * @ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_ZOOM: scale the surface,
 *	preserving aspect ratio, to the largest size that will fit on the output
 * @ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_ZOOM_CROP: scale the surface,
 *	preserving aspect ratio, to fully fill the output cropping if needed
 * @ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_STRETCH: scale the surface to
 *	the size of the output ignoring aspect ratio
 *
 * Hints to indicate to the compositor how to deal with a conflict
 * between the dimensions of the surface and the dimensions of the output.
 * The compositor is free to ignore this parameter.
 */
enum zwp_fullscreen_shell_v1_present_method {
	ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_DEFAULT = 0,
	ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_CENTER = 1,
	ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_ZOOM = 2,
	ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_ZOOM_CROP = 3,
	ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_STRETCH = 4,
};
#endif /* ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_ENUM */

#ifndef ZWP_FULLSCREEN_SHELL_V1_ERROR_ENUM
#define ZWP_FULLSCREEN_SHELL_V1_ERROR_ENUM
/**
 * zwp_fullscreen_shell_v1_error - wl_fullscreen_shell error values
 * @ZWP_FULLSCREEN_SHELL_V1_ERROR_INVALID_METHOD: present_method is not
 *	known
 *
 * These errors can be emitted in response to wl_fullscreen_shell
 * requests
 */
enum zwp_fullscreen_shell_v1_error {
	ZWP_FULLSCREEN_SHELL_V1_ERROR_INVALID_METHOD = 0,
};
#endif /* ZWP_FULLSCREEN_SHELL_V1_ERROR_ENUM */

/**
 * zwp_fullscreen_shell_v1 - Displays a single surface per output
 * @capability: advertises a capability of the compositor
 *
 * Displays a single surface per output.
 *
 * This interface provides a mechanism for a single client to display
 * simple full-screen surfaces. While there technically may be multiple
 * clients bound to this interface, only one of those clients should be
 * shown at a time.
 *
 * To present a surface, the client uses either the present_surface or
 * present_surface_for_mode requests. Presenting a surface takes effect on
 * the next wl_surface.commit. See the individual requests for details
 * about scaling and mode switches.
 *
 * The client can have at most one surface per output at any time.
 * Requesting a surface be presented on an output that already has a
 * surface replaces the previously presented surface. Presenting a null
 * surface removes its content and effectively disables the output. Exactly
 * what happens when an output is "disabled" is compositor-specific. The
 * same surface may be presented on multiple outputs simultaneously.
 *
 * Once a surface is presented on an output, it stays on that output until
 * either the client removes it or the compositor destroys the output. This
 * way, the client can update the output's contents by simply attaching a
 * new buffer.
 *
 * Warning! The protocol described in this file is experimental and
 * backward incompatible changes may be made. Backward compatible changes
 * may be added together with the corresponding interface version bump.
 * Backward incompatible changes are done by bumping the version number in
 * the protocol and interface names and resetting the interface version.
 * Once the protocol is to be declared stable, the 'z' prefix and the
 * version number in the protocol and interface names are removed and the
 * interface version number is reset.
 */
struct zwp_fullscreen_shell_v1_listener {
	/**
	 * capability - advertises a capability of the compositor
	 * @capabilty: (none)
	 *
	 * Advertises a single capability of the compositor.
	 *
	 * When the wl_fullscreen_shell interface is bound, this event is
	 * emitted once for each capability advertised. Valid capabilities
	 * are given by the wl_fullscreen_shell.capability enum. If clients
	 * want to take advantage of any of these capabilities, they should
	 * use a wl_display.sync request immediately after binding to
	 * ensure that they receive all the capability events.
	 */
	void (*capability)(void *data,
			   struct zwp_fullscreen_shell_v1 *zwp_fullscreen_shell_v1,
			   uint32_t capabilty);
};

static inline int
zwp_fullscreen_shell_v1_add_listener(struct zwp_fullscreen_shell_v1 *zwp_fullscreen_shell_v1,
				     const struct zwp_fullscreen_shell_v1_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) zwp_fullscreen_shell_v1,
				     (void (**)(void)) listener, data);
}

#define ZWP_FULLSCREEN_SHELL_V1_RELEASE	0
#define ZWP_FULLSCREEN_SHELL_V1_PRESENT_SURFACE	1
#define ZWP_FULLSCREEN_SHELL_V1_PRESENT_SURFACE_FOR_MODE	2

static inline void
zwp_fullscreen_shell_v1_set_user_data(struct zwp_fullscreen_shell_v1 *zwp_fullscreen_shell_v1, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) zwp_fullscreen_shell_v1, user_data);
}

static inline void *
zwp_fullscreen_shell_v1_get_user_data(struct zwp_fullscreen_shell_v1 *zwp_fullscreen_shell_v1)
{
	return wl_proxy_get_user_data((struct wl_proxy *) zwp_fullscreen_shell_v1);
}

static inline void
zwp_fullscreen_shell_v1_destroy(struct zwp_fullscreen_shell_v1 *zwp_fullscreen_shell_v1)
{
	wl_proxy_destroy((struct wl_proxy *) zwp_fullscreen_shell_v1);
}

static inline void
zwp_fullscreen_shell_v1_release(struct zwp_fullscreen_shell_v1 *zwp_fullscreen_shell_v1)
{
	wl_proxy_marshal((struct wl_proxy *) zwp_fullscreen_shell_v1,
			 ZWP_FULLSCREEN_SHELL_V1_RELEASE);

	wl_proxy_destroy((struct wl_proxy *) zwp_fullscreen_shell_v1);
}

static inline void
zwp_fullscreen_shell_v1_present_surface(struct zwp_fullscreen_shell_v1 *zwp_fullscreen_shell_v1, struct wl_surface *surface, uint32_t method, struct wl_output *output)
{
	wl_proxy_marshal((struct wl_proxy *) zwp_fullscreen_shell_v1,
			 ZWP_FULLSCREEN_SHELL_V1_PRESENT_SURFACE, surface, method, output);
}

static inline struct zwp_fullscreen_shell_mode_feedback_v1 *
zwp_fullscreen_shell_v1_present_surface_for_mode(struct zwp_fullscreen_shell_v1 *zwp_fullscreen_shell_v1, struct wl_surface *surface, struct wl_output *output, int32_t framerate)
{
	struct wl_proxy *feedback;

	feedback = wl_proxy_marshal_constructor((struct wl_proxy *) zwp_fullscreen_shell_v1,
			 ZWP_FULLSCREEN_SHELL_V1_PRESENT_SURFACE_FOR_MODE, &zwp_fullscreen_shell_mode_feedback_v1_interface, surface, output, framerate, NULL);

	return (struct zwp_fullscreen_shell_mode_feedback_v1 *) feedback;
}

struct zwp_fullscreen_shell_mode_feedback_v1_listener {
	/**
	 * mode_successful - mode switch succeeded
	 *
	 * This event indicates that the attempted mode switch operation
	 * was successful. A surface of the size requested in the mode
	 * switch will fill the output without scaling.
	 *
	 * Upon receiving this event, the client should destroy the
	 * wl_fullscreen_shell_mode_feedback object.
	 */
	void (*mode_successful)(void *data,
				struct zwp_fullscreen_shell_mode_feedback_v1 *zwp_fullscreen_shell_mode_feedback_v1);
	/**
	 * mode_failed - mode switch failed
	 *
	 * This event indicates that the attempted mode switch operation
	 * failed. This may be because the requested output mode is not
	 * possible or it may mean that the compositor does not want to
	 * allow it.
	 *
	 * Upon receiving this event, the client should destroy the
	 * wl_fullscreen_shell_mode_feedback object.
	 */
	void (*mode_failed)(void *data,
			    struct zwp_fullscreen_shell_mode_feedback_v1 *zwp_fullscreen_shell_mode_feedback_v1);
	/**
	 * present_cancelled - mode switch cancelled
	 *
	 * This event indicates that the attempted mode switch operation
	 * was cancelled. Most likely this is because the client requested
	 * a second mode switch before the first one completed.
	 *
	 * Upon receiving this event, the client should destroy the
	 * wl_fullscreen_shell_mode_feedback object.
	 */
	void (*present_cancelled)(void *data,
				  struct zwp_fullscreen_shell_mode_feedback_v1 *zwp_fullscreen_shell_mode_feedback_v1);
};

static inline int
zwp_fullscreen_shell_mode_feedback_v1_add_listener(struct zwp_fullscreen_shell_mode_feedback_v1 *zwp_fullscreen_shell_mode_feedback_v1,
						   const struct zwp_fullscreen_shell_mode_feedback_v1_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) zwp_fullscreen_shell_mode_feedback_v1,
				     (void (**)(void)) listener, data);
}

static inline void
zwp_fullscreen_shell_mode_feedback_v1_set_user_data(struct zwp_fullscreen_shell_mode_feedback_v1 *zwp_fullscreen_shell_mode_feedback_v1, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) zwp_fullscreen_shell_mode_feedback_v1, user_data);
}

static inline void *
zwp_fullscreen_shell_mode_feedback_v1_get_user_data(struct zwp_fullscreen_shell_mode_feedback_v1 *zwp_fullscreen_shell_mode_feedback_v1)
{
	return wl_proxy_get_user_data((struct wl_proxy *) zwp_fullscreen_shell_mode_feedback_v1);
}

static inline void
zwp_fullscreen_shell_mode_feedback_v1_destroy(struct zwp_fullscreen_shell_mode_feedback_v1 *zwp_fullscreen_shell_mode_feedback_v1)
{
	wl_proxy_destroy((struct wl_proxy *) zwp_fullscreen_shell_mode_feedback_v1);
}

#ifdef  __cplusplus
}
#endif

#endif
