/* 
 * Copyright (C) 2015-2016 Pier Luigi Fiorini
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GREENISLAND_SCREENCASTER_SERVER_PROTOCOL_H
#define GREENISLAND_SCREENCASTER_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

struct wl_client;
struct wl_resource;

struct greenisland_screencast;
struct greenisland_screencaster;
struct wl_buffer;
struct wl_output;

extern const struct wl_interface greenisland_screencaster_interface;
extern const struct wl_interface greenisland_screencast_interface;

#ifndef GREENISLAND_SCREENCASTER_ERROR_ENUM
#define GREENISLAND_SCREENCASTER_ERROR_ENUM
/**
 * greenisland_screencaster_error - greenisland_screencaster error values
 * @GREENISLAND_SCREENCASTER_ERROR_BAD_OUTPUT: The client passed an
 *	invalid output
 *
 * These errors can be emitted in response to greenisland_screencaster
 * requests.
 */
enum greenisland_screencaster_error {
	GREENISLAND_SCREENCASTER_ERROR_BAD_OUTPUT = 0,
};
#endif /* GREENISLAND_SCREENCASTER_ERROR_ENUM */

/**
 * greenisland_screencaster - capture frames of an output
 * @capture: initiate a screencast
 *
 * This interface is used by programs to record screencasts of an output.
 *
 * The client asks the compositor to capture an output, as a result a
 * greenisland_screencast object is created to drive the screencast.
 *
 * See the documentation of greenisland_screencast for more details.
 */
struct greenisland_screencaster_interface {
	/**
	 * capture - initiate a screencast
	 * @id: (none)
	 * @output: (none)
	 *
	 * This creates a greenisland_screencast object to drive
	 * screencasts. See the documentation of greenisland_screencast for
	 * more details.
	 *
	 * If the output is not valid, a protocol error will be sent.
	 */
	void (*capture)(struct wl_client *client,
			struct wl_resource *resource,
			uint32_t id,
			struct wl_resource *output);
};


#ifndef GREENISLAND_SCREENCAST_ERROR_ENUM
#define GREENISLAND_SCREENCAST_ERROR_ENUM
/**
 * greenisland_screencast_error - greenisland_screencast error values
 * @GREENISLAND_SCREENCAST_ERROR_BAD_BUFFER: Buffer doesn't match
 *	requirements
 *
 * These errors are emitted with the 'failed' event when a capture
 * request fails.
 */
enum greenisland_screencast_error {
	GREENISLAND_SCREENCAST_ERROR_BAD_BUFFER = 0,
};
#endif /* GREENISLAND_SCREENCAST_ERROR_ENUM */

#ifndef GREENISLAND_SCREENCAST_TRANSFORM_ENUM
#define GREENISLAND_SCREENCAST_TRANSFORM_ENUM
/**
 * greenisland_screencast_transform - frame buffer transform
 * @GREENISLAND_SCREENCAST_TRANSFORM_NORMAL: No particular transform has
 *	been applied
 * @GREENISLAND_SCREENCAST_TRANSFORM_Y_INVERTED: Y axis is inverted
 *
 * These values are emitted with the 'frame' event and represent the
 * transform that is applied to the frame buffer.
 */
enum greenisland_screencast_transform {
	GREENISLAND_SCREENCAST_TRANSFORM_NORMAL = 1,
	GREENISLAND_SCREENCAST_TRANSFORM_Y_INVERTED = 2,
};
#endif /* GREENISLAND_SCREENCAST_TRANSFORM_ENUM */

/**
 * greenisland_screencast - frame capture
 * @destroy: destroy greenisland_screencast
 * @record: request a frame to be recorded
 *
 * This interface drives the frame capture into client-allocated buffers.
 * Immediately after this object is created, the compositor sends the
 * 'setup' event with the buffer size. The client uses this information to
 * allocate a buffer to hold the frame image.
 *
 * When the buffer is created, the 'record' request must be invoked each
 * time the client wants to record a frame.
 */
struct greenisland_screencast_interface {
	/**
	 * destroy - destroy greenisland_screencast
	 *
	 * Destroy this greenisland_screencast object. Pending capture
	 * requests will be discarded.
	 */
	void (*destroy)(struct wl_client *client,
			struct wl_resource *resource);
	/**
	 * record - request a frame to be recorded
	 * @buffer: client buffer that will hold the data
	 *
	 * Ask the compositor to record its next frame, saving contents
	 * into the specified buffer. The 'frame' event will be sent after
	 * the frame is recorder. Only one frame will be recorder at a
	 * time, call this request again in order to record another frame.
	 */
	void (*record)(struct wl_client *client,
		       struct wl_resource *resource,
		       struct wl_resource *buffer);
};

#define GREENISLAND_SCREENCAST_SETUP	0
#define GREENISLAND_SCREENCAST_FRAME	1
#define GREENISLAND_SCREENCAST_FAILED	2
#define GREENISLAND_SCREENCAST_CANCELLED	3

#define GREENISLAND_SCREENCAST_SETUP_SINCE_VERSION	1
#define GREENISLAND_SCREENCAST_FRAME_SINCE_VERSION	1
#define GREENISLAND_SCREENCAST_FAILED_SINCE_VERSION	1
#define GREENISLAND_SCREENCAST_CANCELLED_SINCE_VERSION	1

static inline void
greenisland_screencast_send_setup(struct wl_resource *resource_, int32_t width, int32_t height, int32_t stride, int32_t format)
{
	wl_resource_post_event(resource_, GREENISLAND_SCREENCAST_SETUP, width, height, stride, format);
}

static inline void
greenisland_screencast_send_frame(struct wl_resource *resource_, struct wl_resource *buffer, uint32_t time, int32_t transform)
{
	wl_resource_post_event(resource_, GREENISLAND_SCREENCAST_FRAME, buffer, time, transform);
}

static inline void
greenisland_screencast_send_failed(struct wl_resource *resource_, int32_t error, struct wl_resource *buffer)
{
	wl_resource_post_event(resource_, GREENISLAND_SCREENCAST_FAILED, error, buffer);
}

static inline void
greenisland_screencast_send_cancelled(struct wl_resource *resource_, struct wl_resource *buffer)
{
	wl_resource_post_event(resource_, GREENISLAND_SCREENCAST_CANCELLED, buffer);
}

#ifdef  __cplusplus
}
#endif

#endif
